import os
import json
import logging

from sandbox.Core.IPipeline import IPipeline
from sandbox.Core.RegisterPipeline import REGISTERPIPELINE


@REGISTERPIPELINE("reset", "reset work dir using specific env")
class Reset(IPipeline):
    def __init__(self, env_namespace):
        self.env_namespace_ = env_namespace

    @staticmethod
    def declare_args(subparser):
        parser_upload = subparser.add_parser('reset')

        parser_upload.add_argument(
            '-e', '--env',
            required=True,
            help='environment to use')

        parser_upload.add_argument(
            '-o', '--output',
            default="",
            help='output work file')

    def run(self, args) -> int:
        if args.env in self.env_namespace_.keys():
            env = self.env_namespace_[args.env]
            if args.output == "":
                args.output = args.ideas
            os.system(f"rm {args.output}/main.*")
            filename = f"{args.output}/{env.work_filename()}"
            with open(filename, 'w+') as output_file:
                output_file.write(env.template())
                logging.info(f"created {filename} using env {args.env}")
                return 0
        else:
            logging.error(f"{args.env} not in namespace: {json.dumps(list(self.env_namespace_.keys()))}")
        return 1



from abc import ABC, abstractmethod


class IEnvironment(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def template(self) -> str:
        pass

    @abstractmethod
    def execute(self, file_path) -> str:
        pass

    @abstractmethod
    def format(self, file_path) -> str:
        pass

    @abstractmethod
    def is_file_env(self, file_path) -> bool:
        pass

    @abstractmethod
    def extract_meta(self, file_path) -> dict:
        pass

    @abstractmethod
    def work_filename(self) -> str:
        pass



import re
from string import Template

from sandbox.Core.IEnvironment import IEnvironment

class GenericEnvironment(IEnvironment):

    def __init__(self, env):
        self.matcher_ = r"(sandbox_[a-z]+):\s*([a-z]*)"
        self.env_ = env

    def template(self) -> str:
        return self.env_["template"]

    def execute(self, file_path) -> str:
        return Template(self.env_["execution"]).safe_substitute(file_path=file_path)

    def format(self, file_path) -> str:
        return Template(self.env_["format"]).safe_substitute(file_path=file_path)

    def is_file_env(self, file_path) -> bool:
        with open(file_path, "r") as file:
            for line in file:
                find = re.findall(self.matcher_, line)
                for f in find:
                    if f[0] == "sandbox_env":
                        return (f[1] == self.env_["name"])
        return False

    def extract_meta(self, file_path) -> dict:
        ret = dict(
            sandbox_idea="",
            sandbox_name="",
            sandbox_description="",
            sandbox_env=self.env_["name"])
        with open(file_path, "r") as file:
            for line in file:
                find = re.findall(self.matcher_,line)
                for f in find:
                    if f[0] in ret.keys():
                        ret[f[0]] = f[1]
        return ret

    def work_filename(self) -> str:
        return f"main.{self.env_['ext']}"



from abc import ABC, abstractmethod


class IPipeline(ABC):
    def __init__(self):
        pass

    @staticmethod
    @abstractmethod
    def declare_args(subparser):
        pass

    @abstractmethod
    def run(self, args) -> int:
        pass



import os
import toml

from sandbox.Environments.ConfigurableEnvironment import GenericEnvironment

def load_env_namespace(input_conf):
    ret = dict()
    with open(input_conf, "r") as conf:
        tml = toml.loads(conf.read())
        for k in tml:
            tml[k]["name"] = k
            ret[k] = GenericEnvironment(tml[k])
    return ret



import os

from sandbox.Core.FzfHandler import fzf_handler, sniff_dir
from sandbox.Core.IPipeline import IPipeline
from sandbox.Core.RegisterPipeline import REGISTERPIPELINE
from sandbox.Core.ExecuteParentShell import write_on_parent_shell


@REGISTERPIPELINE("execute", "execute env on file")
class Execute(IPipeline):
    def __init__(self, env_namespace):
        self.env_namespace_ = env_namespace

    @staticmethod
    def declare_args(subparser):
        parser_upload = subparser.add_parser('execute')

        parser_upload.add_argument(
            '-p', '--path',
            default="",
            help='work file or ideas directory')

        parser_upload.add_argument(
            '-e', '--env',
            help='force specific env regardless file meta')

    def _execute_file(self, args, file_path):
        env = None

        if args.env and args.env in self.env_namespace_.keys():
            env = self.env_namespace_[args.env]
        else:
            for e in self.env_namespace_.keys():
                if self.env_namespace_[e].is_file_env(file_path):
                    env = self.env_namespace_[e]

        if env:
            os.system(env.format(file_path))
            write_on_parent_shell(env.execute(file_path))
            return 0
        return 1

    def run(self, args) -> int:
        if args.path == "":
            args.path = args.ideas
        if os.path.isfile(args.path):
            return self._execute_file(args, args.path)
        return self._execute_file(args, fzf_handler(sniff_dir(args.path)))



import os
import sys
import pathlib
from pyfzf.pyfzf import FzfPrompt


def sniff_dir(dir):
    dd = pathlib.Path(dir)
    ret = []
    for elem in dd.iterdir():
        if os.path.isfile(elem):
            ret.append(elem)
        else:
            ret += sniff_dir(elem)
    return ret


def fzf_handler(elems):
    fzf = FzfPrompt()
    ret = fzf.prompt(elems)
    if len(ret) == 0:
        sys.exit(0)
    return ret[0]
